@@ -1,9 +1,9 @@
-# This file is part of IFM, the Interactive Fiction Mapper,
-# copyright (C) G. Hutchings 1997-2004.
-#
-# IFM comes with ABSOLUTELY NO WARRANTY.  This is free
-# software, and you are welcome to redistribute it under
-# certain conditions; see the file COPYING for details.
+# This file is part of IFM (Interactive Fiction Mapper), copyright (C)
+# Glenn Hutchings 1997-2008.
+
+# IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
+# are welcome to redistribute it under certain conditions; see the file
+# COPYING for details.
 
 # tkifm -- Tcl/Tk interface to IFM
 
@@ -172,9 +172,6 @@ proc MainWindow {} {
 	-bg $ifm(editbackground) -insertofftime 0 \
 	-insertbackground red
 
-    bind $t <3> "$t scan mark %x %y"
-    bind $t <B3-Motion> "$t scan dragto %x %y"
-
     scrollbar $s -command "$t yview"
 
     pack .edit -expand yes -fill both
@@ -265,8 +262,8 @@ proc DrawMap {num} {
     scrollbar $f.xscroll -command "$c xview" -orient horiz
     scrollbar $f.yscroll -command "$c yview"
 
-    bind $c <3> "$c scan mark %x %y"
-    bind $c <B3-Motion> "$c scan dragto %x %y"
+    bind $c <1> "$c scan mark %x %y"
+    bind $c <B1-Motion> "$c scan dragto %x %y"
 
     grid $c -in $f -row 0 -column 0 \
 	    -rowspan 1 -columnspan 1 -sticky nsew
@@ -589,9 +586,6 @@ proc TextWindow {title w text width height} {
     $t insert end $text
     $t configure -state disabled
 
-    bind $t <3> "$t scan mark %x %y"
-    bind $t <B3-Motion> "$t scan dragto %x %y"
-
     # Add 'dismiss' button.
     set b $w.bye
     button $b -text "Dismiss" -command "destroy $w"
