@@ -1,10 +1,10 @@
 /*
- * This file is part of IFM, the Interactive Fiction Mapper,
- * copyright (C) G. Hutchings 1997-2004.
+ * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
+ * Glenn Hutchings 1997-2008.
  *
- * IFM comes with ABSOLUTELY NO WARRANTY.  This is free
- * software, and you are welcome to redistribute it under
- * certain conditions; see the file COPYING for details.
+ * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
+ * are welcome to redistribute it under certain conditions; see the file
+ * COPYING for details.
  */
 
 /* Main function header */
@@ -20,7 +20,6 @@ extern vlist *ifm_styles;
 
 /* Advertised functions */
 extern int parse_input(char *file, int libflag, int required);
-extern void switch_file(char *file, int line, int flag);
 extern void yyerror(char *msg);
 extern void err(char *fmt, ...);
 extern void warn(char *fmt, ...);
