@@ -1,10 +1,10 @@
 /*
- * This file is part of IFM, the Interactive Fiction Mapper,
- * copyright (C) G. Hutchings 1997-2004.
+ * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
+ * Glenn Hutchings 1997-2008.
  *
- * IFM comes with ABSOLUTELY NO WARRANTY.  This is free
- * software, and you are welcome to redistribute it under
- * certain conditions; see the file COPYING for details.
+ * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
+ * are welcome to redistribute it under certain conditions; see the file
+ * COPYING for details.
  */
 
 /* Path building functions */
@@ -26,7 +26,7 @@
 
 #define NODE(room)           vh_sgetref(room, "NODE")
 
-#define PATH_LIST(r1, r2)    vg_path_list(graph, NODE(r1), NODE(r2))
+#define PATH_NODES(r1, r2)   vg_path_nodes(graph, NODE(r1), NODE(r2))
 #define PATH_INFO(r1, r2)    vg_path_info(graph, NODE(r1), NODE(r2))
 #define PATH_LENGTH(r1, r2)  ((int) vg_path_length(graph, NODE(r1), NODE(r2)))
 
@@ -64,7 +64,7 @@ connect_rooms(void)
     vhash *room, *link, *join, *reach, *from, *to;
     vlist *cmdfrom, *cmdto, *list;
     char *cmd, id[10];
-    vscalar *elt;
+    viter iter;
 
     solver_msg(0, "Connecting rooms...");
 
@@ -72,18 +72,20 @@ connect_rooms(void)
     graph = vg_create();
 
     /* Create room nodes */
-    vl_foreach(elt, rooms) {
-        room = vs_pget(elt);
+    v_iterate(rooms, iter) {
+        room = vl_iter_pval(iter);
         sprintf(id, "R%d", ++num);
         vh_sstore(room, "NODE", id);
         vg_node_pstore(graph, id, room);
     }
 
     /* Build link connections */
-    vl_foreach(elt, links) {
-        link = vs_pget(elt);
+    v_iterate(links, iter) {
+        link = vl_iter_pval(iter);
+
         if (vh_iget(link, "NOLINK"))
             continue;
+
         if (vh_iget(link, "NOPATH"))
             continue;
 
@@ -179,8 +181,8 @@ connect_rooms(void)
     }
 
     /* Build join connections */
-    vl_foreach(elt, joins) {
-        join = vs_pget(elt);
+    v_iterate(joins, iter) {
+        join = vl_iter_pval(iter);
         if (vh_iget(join, "NOPATH"))
             continue;
 
@@ -347,14 +349,14 @@ get_path(vhash *step, vhash *room)
     char *node, *next;
     int i, len, found;
     vhash *reach;
-    vscalar *elt;
+    viter iter;
 
     /* Build path */
     list = vh_pget(step, "PATH");
     vh_delete(step, "PATH");
 
     if (list == NULL)
-        list = PATH_LIST(path_room, room);
+        list = PATH_NODES(path_room, room);
 
     if (list == NULL)
         return NULL;
@@ -373,8 +375,8 @@ get_path(vhash *step, vhash *room)
         rlist = vg_link_pget(graph, node, next);
         found = 0;
 
-        vl_foreach(elt, rlist) {
-            reach = vs_pget(elt);
+        v_iterate(rlist, iter) {
+            reach = vl_iter_pval(iter);
             if (!found && vh_iget(reach, "USE")) {
                 vl_ppush(path, reach);
                 found++;
@@ -394,9 +396,9 @@ init_path(vhash *room)
 {
     vhash *step, *item, *taskroom;
     int len, blockable, offset;
-    vscalar *elt;
     vlist *list;
     double dist;
+    viter i, j;
 
     /* Only need update if room changed, or path modified */
     if (room == path_room && !path_modify)
@@ -406,8 +408,8 @@ init_path(vhash *room)
     path_modify = 0;
 
     /* Deal with tasks that need droppable items */
-    vl_foreach(elt, tasklist) {
-        step = vs_pget(elt);
+    v_iterate(tasklist, i) {
+        step = vl_iter_pval(i);
         if (vh_iget(step, "DONE"))
             continue;
 
@@ -418,10 +420,12 @@ init_path(vhash *room)
             continue;
 
         blockable = 0;
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, j) {
+            item = vl_iter_pval(j);
+
             if (!vh_iget(item, "TAKEN"))
                 continue;
+
             if (!vh_iget(item, "LEAVE"))
                 continue;
 
@@ -454,8 +458,8 @@ init_path(vhash *room)
     solver_msg(2, "updated path cache (max dist %g)", dist);
 
     /* Record distance of each task */
-    vl_foreach(elt, tasklist) {
-        step = vs_pget(elt);
+    v_iterate(tasklist, i) {
+        step = vl_iter_pval(i);
 
         if (vh_iget(step, "BLOCK"))
             len = vh_iget(step, "SORT");
@@ -471,13 +475,13 @@ init_path(vhash *room)
     }
 
     /* Sort tasks according to distance */
-    vl_sort_inplace(tasklist, sort_tasks);
+    vl_sort(tasklist, sort_tasks);
 
     if (TASK_VERBOSE) {
         vhash *room;
 
-        vl_foreach(elt, tasklist) {
-            step = vs_pget(elt);
+        v_iterate(tasklist, i) {
+            step = vl_iter_pval(i);
 
             if (vh_iget(step, "DONE") || vh_iget(step, "IGNORED"))
                 continue;
@@ -502,9 +506,9 @@ static void
 link_rooms(vhash *from, vhash *to, vhash *reach)
 {
     char *fnode, *tnode;
-    vscalar *elt;
     vlist *list;
     vhash *relt;
+    viter iter;
     int len;
 
     fnode = vh_sgetref(from, "NODE");
@@ -518,8 +522,8 @@ link_rooms(vhash *from, vhash *to, vhash *reach)
 
     /* Check lengths agree (unfortunate limitation) */
     len = vh_iget(reach, "LEN");
-    vl_foreach(elt, list) {
-        relt = vs_pget(elt);
+    v_iterate(list, iter) {
+        relt = vl_iter_pval(iter);
         if (len != vh_iget(relt, "LEN"))
             err("links between '%s' and '%s' have differing lengths",
                 vh_sgetref(from, "DESC"), vh_sgetref(to, "DESC"));
@@ -555,7 +559,7 @@ reachable_rooms(vhash *room)
     static vlist *list = NULL;
     vlist *rlist, *tlist;
     char *node, *next;
-    vscalar *elt;
+    viter i, j;
 
     if (list == NULL)
         list = vl_create();
@@ -565,11 +569,11 @@ reachable_rooms(vhash *room)
     node = NODE(room);
     tlist = vg_node_to(graph, node);
 
-    vl_foreach(elt, tlist) {
-        next = vs_sgetref(elt);
+    v_iterate(tlist, i) {
+        next = vl_iter_svalref(i);
         rlist = vg_link_pget(graph, node, next);
-        vl_foreach(elt, rlist)
-            vl_ppush(list, vs_pget(elt));
+        v_iterate(rlist, j)
+            vl_ppush(list, vl_iter_pval(j));
     }
 
     vl_destroy(tlist);
@@ -610,20 +614,20 @@ use_link(char *fnode, char *tnode, vscalar *s)
 {
     vhash *reach, *item, *task, *tstep, *block, *room;
     vlist *list, *rlist;
-    vscalar *elt;
+    viter i, j;
     int flag;
 
     /* Loop over all reach elements of this link */
     rlist = vs_pget(s);
-    vl_foreach(elt, rlist) {
-        reach = vs_pget(elt);
+    v_iterate(rlist, i) {
+        reach = vl_iter_pval(i);
         vh_istore(reach, "USE", 0);
         flag = 1;
 
         /* Check items needed by task don't have to be left */
         if (path_task != NULL && (list = vh_pget(reach, "LEAVE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 block = vh_pget(item, "BLOCK");
                 if (block != NULL && block == path_task) {
                     if (TASK_VERBOSE) {
@@ -634,7 +638,6 @@ use_link(char *fnode, char *tnode, vscalar *s)
                                vh_sgetref(item, "DESC"));
                     }
 
-                    vl_break(list);
                     flag = 0;
                     break;
                 }
@@ -646,8 +649,8 @@ use_link(char *fnode, char *tnode, vscalar *s)
 
         /* Check required items have been obtained */
         if ((list = vh_pget(reach, "NEED")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 if (!vh_iget(item, "TAKEN")) {
                     if (TASK_VERBOSE) {
                         indent(4 - vg_caching());
@@ -657,7 +660,6 @@ use_link(char *fnode, char *tnode, vscalar *s)
                                vh_sgetref(item, "DESC"));
                     }
 
-                    vl_break(list);
                     flag = 0;
                     break;
                 }
@@ -669,8 +671,8 @@ use_link(char *fnode, char *tnode, vscalar *s)
 
         /* Check status of required tasks */
         if ((list = vh_pget(reach, "BEFORE")) != NULL) {
-            vl_foreach(elt, list) {
-                task = vs_pget(elt);
+            v_iterate(list, j) {
+                task = vl_iter_pval(j);
                 tstep = vh_pget(task, "STEP");
                 if (vh_iget(tstep, "DONE")) {
                     if (TASK_VERBOSE) {
@@ -681,7 +683,6 @@ use_link(char *fnode, char *tnode, vscalar *s)
                                vh_sgetref(task, "DESC"));
                     }
 
-                    vl_break(list);
                     flag = 0;
                     break;
                 }
@@ -692,8 +693,8 @@ use_link(char *fnode, char *tnode, vscalar *s)
         }
 
         if ((list = vh_pget(reach, "AFTER")) != NULL) {
-            vl_foreach(elt, list) {
-                task = vs_pget(elt);
+            v_iterate(list, j) {
+                task = vl_iter_pval(j);
                 tstep = vh_pget(task, "STEP");
                 if (!vh_iget(tstep, "DONE")) {
                     if (TASK_VERBOSE) {
@@ -704,7 +705,6 @@ use_link(char *fnode, char *tnode, vscalar *s)
                                vh_sgetref(task, "DESC"));
                     }
 
-                    vl_break(list);
                     flag = 0;
                     break;
                 }
@@ -716,7 +716,6 @@ use_link(char *fnode, char *tnode, vscalar *s)
 
         /* Flag it as usable */
         vh_istore(reach, "USE", 1);
-        vl_break(rlist);
         return 1;
     }
 
@@ -728,15 +727,15 @@ static int
 use_node(char *node, vscalar *s, double dist)
 {
     vhash *room, *item, *task, *tstep, *block;
-    vscalar *elt;
     vlist *list;
+    viter iter;
 
     room = vs_pget(s);
 
     /* Check items needed by task don't have to be left */
     if (path_task != NULL && (list = vh_pget(room, "LEAVE")) != NULL) {
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, iter) {
+            item = vl_iter_pval(iter);
             block = vh_pget(item, "BLOCK");
             if (block != NULL && block == path_task) {
                 if (TASK_VERBOSE) {
@@ -746,7 +745,6 @@ use_node(char *node, vscalar *s, double dist)
                            vh_sgetref(item, "DESC"));
                 }
 
-                vl_break(list);
                 return 0;
             }
         }
@@ -754,8 +752,8 @@ use_node(char *node, vscalar *s, double dist)
 
     /* Check for required items */
     if ((list = vh_pget(room, "NEED")) != NULL) {
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, iter) {
+            item = vl_iter_pval(iter);
             if (!vh_iget(item, "TAKEN")) {
                 if (TASK_VERBOSE) {
                     indent(4 - vg_caching());
@@ -764,7 +762,6 @@ use_node(char *node, vscalar *s, double dist)
                            vh_sgetref(item, "DESC"));
                 }
 
-                vl_break(list);
                 return 0;
             }
         }
@@ -772,8 +769,8 @@ use_node(char *node, vscalar *s, double dist)
 
     /* Check status of required tasks */
     if ((list = vh_pget(room, "BEFORE")) != NULL) {
-        vl_foreach(elt, list) {
-            task = vs_pget(elt);
+        v_iterate(list, iter) {
+            task = vl_iter_pval(iter);
             tstep = vh_pget(task, "STEP");
             if (vh_iget(tstep, "DONE")) {
                 if (TASK_VERBOSE) {
@@ -783,15 +780,14 @@ use_node(char *node, vscalar *s, double dist)
                            vh_sgetref(task, "DESC"));
                 }
 
-                vl_break(list);
                 return 0;
             }
         }
     }
 
     if ((list = vh_pget(room, "AFTER")) != NULL) {
-        vl_foreach(elt, list) {
-            task = vs_pget(elt);
+        v_iterate(list, iter) {
+            task = vl_iter_pval(iter);
             tstep = vh_pget(task, "STEP");
             if (!vh_iget(tstep, "DONE")) {
                 if (TASK_VERBOSE) {
@@ -801,7 +797,6 @@ use_node(char *node, vscalar *s, double dist)
                            vh_sgetref(task, "DESC"));
                 }
 
-                vl_break(list);
                 return 0;
             }
         }
