@@ -1,10 +1,10 @@
 /*
- * This file is part of IFM, the Interactive Fiction Mapper,
- * copyright (C) G. Hutchings 1997-2004.
+ * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
+ * Glenn Hutchings 1997-2008.
  *
- * IFM comes with ABSOLUTELY NO WARRANTY.  This is free
- * software, and you are welcome to redistribute it under
- * certain conditions; see the file COPYING for details.
+ * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
+ * are welcome to redistribute it under certain conditions; see the file
+ * COPYING for details.
  */
 
 /* Recording output driver */
@@ -39,12 +39,12 @@ static void putline(char *str);
 void
 rec_task_entry(vhash *task)
 {
-    vscalar *elt;
     vlist *cmds;
+    viter iter;
 
     if ((cmds = vh_pget(task, "CMD")) != NULL) {
-        vl_foreach(elt, cmds)
-            putline(vs_sgetref(elt));
+        v_iterate(cmds, iter)
+            putline(vl_iter_svalref(iter));
     } else {
         putline(vh_sgetref(task, "DESC"));
     }
@@ -53,8 +53,8 @@ rec_task_entry(vhash *task)
 static void
 putline(char *str)
 {
-    str = var_subst(str);
     while (*str != '\0')
         putchar(toupper(*str++));
+
     putchar('\n');
 }
