@@ -1,10 +1,10 @@
 /*
- * This file is part of IFM, the Interactive Fiction Mapper,
- * copyright (C) G. Hutchings 1997-2004.
+ * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
+ * Glenn Hutchings 1997-2008.
  *
- * IFM comes with ABSOLUTELY NO WARRANTY.  This is free
- * software, and you are welcome to redistribute it under
- * certain conditions; see the file COPYING for details.
+ * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
+ * are welcome to redistribute it under certain conditions; see the file
+ * COPYING for details.
  */
 
 /* Task building functions */
@@ -26,16 +26,31 @@
 #include "ifm-util.h"
 #include "ifm-vars.h"
 
-#define TS_INVALID 0
-#define TS_IGNORED 1
-#define TS_UNSAFE  2
-#define TS_SAFE    3
+#define MOVETO(room) do {                               \
+    location = room;                                    \
+    if (location != NULL)                               \
+        location_desc = vh_sgetref(location, "DESC");   \
+    else                                                \
+        location_desc = "nowhere";                      \
+} while (0)
+
+/* Task step attributes */
+static char *taskattr[] = {
+    "TAG", "STYLE", "CMD", "DROPALL", "HIDDEN", "DO", "DROP", "DROPROOM",
+    "DROPUNTIL", "GET", "GIVE", "GOTO", "LOSE", "NEED", NULL
+};
+
+/* Task step flags */
+enum {
+    TS_INVALID, TS_IGNORED, TS_UNSAFE, TS_SAFE
+};
 
 /* Task step list */
 vlist *tasklist = NULL;
 
 /* Current location */
 static vhash *location = NULL;
+static char *location_desc = "nowhere";
 
 /* Control variables */
 static int all_tasks_safe = 0;
@@ -46,26 +61,28 @@ static void add_task(vhash *task);
 static int do_task(vhash *task, int print, int recurse);
 static void drop_item(vhash *item, vhash *room, vlist *until, int print);
 static void filter_tasks(int print);
+static vhash *first_task(vhash *step);
 static void goto_room(vhash *task);
 static void invert_items(vhash *obj, char *attr);
+static void mark_finishing(char *action, char *otype, char *varname,
+                           vhash *table);
 static vhash *new_task(int type, vhash *data);
 static void order_tasks(vhash *before, vhash *after);
 static int task_status(vhash *room, vhash *step);
 static int want_item(vhash *item);
 static void warn_failure(void);
 
-/* Add a task to the task list */
+/* Add a task to the task list (if not already there) */
 static void
 add_task(vhash *task)
 {
-    if (vh_exists(task, "DEPEND"))
-        return;
+    if (!vh_exists(task, "DEPEND")) {
+        if (tasklist == NULL)
+            tasklist = vl_create();
 
-    if (tasklist == NULL)
-        tasklist = vl_create();
-
-    vl_ppush(tasklist, task);
-    vh_pstore(task, "DEPEND", vl_create());
+        vl_ppush(tasklist, task);
+        vh_pstore(task, "DEPEND", vl_create());
+    }
 }
 
 /* Check task list for cyclic dependencies */
@@ -74,10 +91,10 @@ check_cycles(void)
 {
     vlist *tsort, *cycles;
     char *line, *id;
-    vscalar *elt;
     vhash *step;
     vlist *list;
     V_BUF_DECL;
+    viter i, j;
     vgraph *g;
     int count;
 
@@ -96,11 +113,15 @@ check_cycles(void)
     }
 
     /* Cycles found -- give error and die */
+    V_BUF_INIT;
     cycles = vg_tsort_cycles();
-    vl_foreach(elt, cycles) {
-        list = vs_pget(elt);
-        vl_foreach(elt, list) {
-            id = vs_sgetref(elt);
+
+    v_iterate(cycles, i) {
+        vscalar *elt;
+        list = vl_iter_pval(i);
+        v_iterate(list, j) {
+            elt = vl_iter_val(j);
+            id = vl_iter_svalref(j);
             step = vg_node_pget(g, id);
             vs_sstore(elt, vh_sgetref(step, "DESC"));
         }
@@ -109,8 +130,8 @@ check_cycles(void)
         list = vl_filltext(line, 65);
 
         V_BUF_ADD("   cycle:\n");
-        vl_foreach(elt, list)
-            V_BUF_ADD1("      %s\n", vs_sgetref(elt));
+        v_iterate(list, j)
+            V_BUF_ADD1("      %s\n", vl_iter_svalref(j));
     }
 
     count = vl_length(cycles);
@@ -125,13 +146,14 @@ do_task(vhash *task, int print, int recurse)
     int scoretask = 1, score, filter = 0;
     vhash *item, *room, *otask, *step;
     vlist *list, *until;
-    vscalar *elt;
+    viter iter;
 
     if (vh_iget(task, "DONE"))
         return 1;
 
     /* Do the task */
     switch (vh_iget(task, "TYPE")) {
+
     case T_GET:
         item = vh_pget(task, "DATA");
         if (vh_exists(item, "TAKEN"))
@@ -147,6 +169,7 @@ do_task(vhash *task, int print, int recurse)
             add_attr(task, "NOTE", "Not used for anything yet");
 
         break;
+
     case T_DROP:
         item = vh_pget(task, "DATA");
         vh_istore(item, "TAKEN", 0);
@@ -155,10 +178,11 @@ do_task(vhash *task, int print, int recurse)
             print = 0;
 
         break;
+
     case T_USER:
         if ((list = vh_pget(task, "GIVE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, iter) {
+                item = vl_iter_pval(iter);
                 add_attr(task, "NOTE", "Gives %s", vh_sgetref(item, "DESC"));
                 if (!vh_iget(item, "USED"))
                     add_attr(task, "NOTE", "Not used for anything yet");
@@ -174,6 +198,7 @@ do_task(vhash *task, int print, int recurse)
 
         filter = 1;
         break;
+
     case T_GOTO:
         print = 0;
         break;
@@ -192,8 +217,8 @@ do_task(vhash *task, int print, int recurse)
 
     /* Get any given items */
     if ((list = vh_pget(task, "GIVE")) != NULL) {
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, iter) {
+            item = vl_iter_pval(iter);
 
             if (!vh_exists(item, "TAKEN")) {
                 score = vh_iget(task, "SCORE") + vh_iget(item, "SCORE");
@@ -207,8 +232,8 @@ do_task(vhash *task, int print, int recurse)
 
     /* Lose any lost items */
     if ((list = vh_pget(task, "LOSE")) != NULL) {
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, iter) {
+            item = vl_iter_pval(iter);
             solver_msg(3, "lose item: %s", vh_sgetref(item, "DESC"));
             vh_istore(item, "TAKEN", 0);
         }
@@ -221,16 +246,16 @@ do_task(vhash *task, int print, int recurse)
 
         until = vh_pget(task, "DROPUNTIL");
 
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, iter) {
+            item = vl_iter_pval(iter);
             drop_item(item, room, until, 0);
         }
     }
 
     /* Do any other tasks */
     if ((list = vh_pget(task, "DO")) != NULL) {
-        vl_foreach(elt, list) {
-            otask = vs_pget(elt);
+        v_iterate(list, iter) {
+            otask = vl_iter_pval(iter);
             step = vh_pget(otask, "STEP");
             if (vh_iget(step, "DONE"))
                 continue;
@@ -253,7 +278,7 @@ do_task(vhash *task, int print, int recurse)
     /* Teleport to new location if required */
     if ((room = vh_pget(task, "GOTO")) != NULL) {
         solver_msg(2, "goto room: %s", vh_sgetref(room, "DESC"));
-        location = room;
+        MOVETO(room);
     }
 
     /* Flag path modification if required */
@@ -268,7 +293,7 @@ do_task(vhash *task, int print, int recurse)
     if (vh_iget(task, "FINISH"))
         return 0;
 
-    if (vh_iget(location, "FINISH"))
+    if (location != NULL && vh_iget(location, "FINISH"))
         return 0;
 
     return 1;
@@ -279,8 +304,8 @@ static void
 drop_item(vhash *item, vhash *room, vlist *until, int print)
 {
     vhash *tstep, *step, *task;
-    vscalar *elt;
     vlist *list;
+    viter iter;
 
     /* Do nothing if not carrying it */
     if (!vh_iget(item, "TAKEN"))
@@ -296,8 +321,8 @@ drop_item(vhash *item, vhash *room, vlist *until, int print)
 
     /* Add new task dependencies (if any) */
     if (until != NULL) {
-        vl_foreach(elt, until) {
-            task = vs_pget(elt);
+        v_iterate(until, iter) {
+            task = vl_iter_pval(iter);
             tstep = vh_pget(task, "STEP");
             if (!vh_iget(tstep, "DONE"))
                 order_tasks(tstep, step);
@@ -305,8 +330,8 @@ drop_item(vhash *item, vhash *room, vlist *until, int print)
     }
 
     if ((list = vh_pget(item, "TASKS")) != NULL) {
-        vl_foreach(elt, list) {
-            tstep = vs_pget(elt);
+        v_iterate(list, iter) {
+            tstep = vl_iter_pval(iter);
             if (!vh_iget(tstep, "DONE"))
                 order_tasks(step, tstep);
         }
@@ -326,15 +351,15 @@ filter_tasks(int print)
     int canfilter, filter, filtered, numfiltered = 0, alldone;
     vhash *task, *otask, *item, *step;
     char *reason;
-    vscalar *elt;
     vlist *list;
+    viter i, j;
 
     /* Loop until no more filtering possible */
     do {
         filtered = 0;
 
-        vl_foreach(elt, tasklist) {
-            task = vs_pget(elt);
+        v_iterate(tasklist, i) {
+            task = vl_iter_pval(i);
 
             /* By default, can't filter */
             canfilter = 0;
@@ -352,8 +377,8 @@ filter_tasks(int print)
             if (filter && (list = vh_pget(task, "ALLOW")) != NULL) {
                 alldone = 1;
 
-                vl_foreach(elt, list) {
-                    otask = vs_pget(elt);
+                v_iterate(list, j) {
+                    otask = vl_iter_pval(j);
                     if (!vh_iget(otask, "DONE"))
                         alldone = 0;
                 }
@@ -368,8 +393,8 @@ filter_tasks(int print)
                 canfilter = 1;
                 reason = "items already carried";
 
-                vl_foreach(elt, list) {
-                    item = vs_pget(elt);
+                v_iterate(list, j) {
+                    item = vl_iter_pval(j);
                     if (!vh_exists(item, "TAKEN"))
                         alldone = 0;
                 }
@@ -384,8 +409,8 @@ filter_tasks(int print)
                 canfilter = 1;
                 reason = "items already carried";
 
-                vl_foreach(elt, list) {
-                    item = vs_pget(elt);
+                v_iterate(list, j) {
+                    item = vl_iter_pval(j);
                     if (!vh_exists(item, "TAKEN"))
                         alldone = 0;
                 }
@@ -399,8 +424,8 @@ filter_tasks(int print)
                 alldone = 1;
                 reason = "tasks already done";
 
-                vl_foreach(elt, list) {
-                    otask = vs_pget(elt);
+                v_iterate(list, j) {
+                    otask = vl_iter_pval(j);
                     step = vh_pget(otask, "STEP");
                     if (!vh_iget(step, "DONE"))
                         alldone = 0;
@@ -433,13 +458,29 @@ filter_tasks(int print)
         modify_path(print);
 }
 
+/* Return first task in a task's follow-chain */
+static vhash *
+first_task(vhash *step)
+{
+    vhash *first, *prev;
+
+    if ((first = vh_pget(step, "PREV")) != NULL) {
+        while ((prev = vh_pget(first, "PREV")) != NULL)
+            first = prev;
+    } else {
+        first = step;
+    }
+
+    return first;
+}
+
 /* Move to a room to do something */
 static void
 goto_room(vhash *task)
 {
     vhash *item, *reach, *mtask, *last, *room;
     vlist *path, *list;
-    vscalar *elt;
+    viter i, j;
 
     /* See if movement is required */
     if ((room = vh_pget(task, "ROOM")) == NULL)
@@ -451,21 +492,21 @@ goto_room(vhash *task)
     path = get_path(task, room);
     last = location;
 
-    vl_foreach(elt, path) {
-        reach = vs_pget(elt);
+    v_iterate(path, i) {
+        reach = vl_iter_pval(i);
         room = vh_pget(reach, "TO");
 
         /* Leave items behind if required */
         if ((list = vh_pget(reach, "LEAVE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 drop_item(item, last, NULL, 1);
             }
         }
 
         if ((list = vh_pget(room, "LEAVE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 drop_item(item, last, NULL, 1);
             }
         }
@@ -490,7 +531,7 @@ goto_room(vhash *task)
         last = room;
     }
 
-    location = room;
+    MOVETO(room);
 }
 
 /* Invert an item list if required */
@@ -499,45 +540,73 @@ invert_items(vhash *obj, char *attr)
 {
     vlist *list, *newlist = NULL;
     static int num = 0;
-    vscalar *elt;
     vhash *item;
+    viter iter;
 
     list = vh_pget(obj, attr);
     num++;
 
     if (list != NULL) {
-        vl_foreach(elt, list) {
-            item = vs_pget(elt);
+        v_iterate(list, iter) {
+            item = vl_iter_pval(iter);
             vh_istore(item, "INVERT", num);
         }
     }
 
-    vl_foreach(elt, items) {
-        item = vs_pget(elt);
+    v_iterate(items, iter) {
+        item = vl_iter_pval(iter);
         if (vh_iget(item, "INVERT") == num)
             continue;
+
         if (newlist == NULL)
             newlist = vl_create();
+
         vl_ppush(newlist, item);
     }
 
     vh_pstore(obj, attr, newlist);
 }
 
+/* Mark an object as finishing the game */
+static void
+mark_finishing(char *action, char *otype, char *varname, vhash *table)
+{
+    char *value, *tag, *name;
+    vlist *tags;
+    vhash *obj;
+    viter i;
+
+    value = var_string(varname);
+    tags = vl_split(value, ",");
+
+    v_iterate(tags, i) {
+        tag = vl_iter_sval(i);
+        if ((obj = vh_pget(table, tag)) != NULL) {
+            name = vh_sgetref(obj, "DESC");
+            solver_msg(2, "%s %s '%s'", action, otype, name);
+            vh_istore(obj, "FINISH", 1);
+        } else {
+            err("%s tag '%s' not defined", otype, tag);
+        }
+    }
+
+    vl_destroy(tags);
+}
+
 /* Create and return a new task step */
 static vhash *
 new_task(int type, vhash *data)
 {
     char *desc = vh_sgetref(data, "DESC");
-    int score = vh_iget(data, "SCORE");
+    int i, score = vh_iget(data, "SCORE");
     static int taskid = 0;
     vhash *room, *step;
+    vscalar *val;
     V_BUF_DECL;
 
     step = vh_create();
     vh_istore(step, "TYPE", type);
-    if (data != NULL)
-        vh_pstore(step, "DATA", data);
+    vh_pstore(step, "DATA", data);
 
     switch (type) {
     case T_MOVE:
@@ -561,26 +630,10 @@ new_task(int type, vhash *data)
         V_BUF_SET(desc);
         room = vh_pget(data, "ROOM");
 
-        if (vh_exists(data, "TAG"))
-            vh_sstore(step, "TAG", vh_sgetref(data, "TAG"));
+        for (i = 0; taskattr[i] != NULL; i++)
+            if ((val = vh_get(data, taskattr[i])) != NULL)
+                vh_store(step, taskattr[i], vs_copy(val));
 
-        if (vh_exists(data, "STYLE"))
-            vh_pstore(step, "STYLE", vh_pget(data, "STYLE"));
-
-        if (vh_exists(data, "CMD"))
-            vh_pstore(step, "CMD", vh_pget(data, "CMD"));
-
-        vh_istore(step, "DROPALL", vh_iget(data, "DROPALL"));
-        vh_istore(step, "HIDDEN", vh_iget(data, "HIDDEN"));
-        vh_pstore(step, "DO", vh_pget(data, "DO"));
-        vh_pstore(step, "DROP", vh_pget(data, "DROP"));
-        vh_pstore(step, "DROPROOM", vh_pget(data, "DROPROOM"));
-        vh_pstore(step, "DROPUNTIL", vh_pget(data, "DROPUNTIL"));
-        vh_pstore(step, "GET", vh_pget(data, "GET"));
-        vh_pstore(step, "GIVE", vh_pget(data, "GIVE"));
-        vh_pstore(step, "GOTO", vh_pget(data, "GOTO"));
-        vh_pstore(step, "LOSE", vh_pget(data, "LOSE"));
-        vh_pstore(step, "NEED", vh_pget(data, "NEED"));
         break;
     default:
         fatal("internal: unknown task type");
@@ -610,32 +663,32 @@ static void
 order_tasks(vhash *before, vhash *after)
 {
     vlist *allow, *depend;
-    vhash *start = after;
+    int count = 0;
+    vhash *step;
 
+    /* Add the tasks */
     add_task(before);
     add_task(after);
 
-    if (before == after)
-        return;
+    if (before != after) {
+        /* The 'before' task allows the 'after' one to be done */
+        add_list(before, "ALLOW", after);
 
-    if ((allow = vh_pget(before, "ALLOW")) == NULL) {
-        allow = vl_create();
-        vh_pstore(before, "ALLOW", allow);
-    }
+        /* The 'after' task (and previous ones in its follow-chain)
+         * depends on the 'before' one */
+        for (step = after; step != NULL; step = vh_pget(step, "PREV")) {
+            if (step != before) {
+                add_list(step, "DEPEND", before);
+                solver_msg(2, "task order: do '%s' before '%s'",
+                           vh_sgetref(before, "DESC"),
+                           vh_sgetref(step, "DESC"));
+            }
 
-    vl_ppush(allow, after);
-
-    do {
-        if (after != before) {
-            depend = vh_pget(after, "DEPEND");
-            vl_ppush(depend, before);
-            solver_msg(2, "task order: do '%s' before '%s'",
-                       vh_sgetref(before, "DESC"),
-                       vh_sgetref(after, "DESC"));
+            /* Infinite-loop paranoia check */
+            if (step == after && count++ > 0)
+                break;
         }
-
-        after = vh_pget(after, "PREV");
-    } while (after != NULL && after != start);
+    }
 }
 
 /* Return a task required by a given task, if any */
@@ -644,16 +697,15 @@ require_task(vhash *step)
 {
     vhash *before;
     vlist *depend;
-    vscalar *elt;
+    viter iter;
 
-    if ((depend = vh_pget(step, "DEPEND")) != NULL) {
-        vl_foreach(elt, depend) {
-            before = vs_pget(elt);
-            if (!vh_iget(before, "DONE")) {
-                vl_break(depend);
-                return before;
-            }
-        }
+    if ((depend = vh_pget(step, "DEPEND")) == NULL)
+        return NULL;
+
+    v_iterate(depend, iter) {
+        before = vl_iter_pval(iter);
+        if (!vh_iget(before, "DONE"))
+            return before;
     }
 
     return NULL;
@@ -664,17 +716,23 @@ void
 setup_tasks(void)
 {
     vhash *task, *otask, *get, *after, *item, *tstep, *room, *reach;
+    vhash *step, *istep, *oitem, *first;
     vlist *list, *itasks, *rlist;
-    vhash *step, *istep, *oitem;
     int flagged = 1;
-    vscalar *elt;
+    viter i, j, k;
     char *msg;
 
     solver_msg(0, "\nSetting up tasks...");
 
+    /* Flag any extra rooms/items/tasks as finishing things */
+    solver_msg(1, "Marking extra events that finish the game");
+    mark_finishing("entering", "room", "finish_room", roomtags);
+    mark_finishing("obtaining", "item", "finish_item", itemtags);
+    mark_finishing("doing", "task", "finish_task", tasktags);
+
     /* Create 'goto room' steps */
-    vl_foreach(elt, rooms) {
-        room = vs_pget(elt);
+    v_iterate(rooms, i) {
+        room = vl_iter_pval(i);
         step = new_task(T_GOTO, room);
         vh_pstore(room, "STEP", step);
         if (vh_iget(room, "SCORE") > 0 || vh_iget(room, "FINISH"))
@@ -682,8 +740,8 @@ setup_tasks(void)
     }
 
     /* Create 'get item' steps */
-    vl_foreach(elt, items) {
-        item = vs_pget(elt);
+    v_iterate(items, i) {
+        item = vl_iter_pval(i);
         step = new_task(T_GET, item);
         vh_pstore(item, "STEP", step);
         if (vh_iget(item, "SCORE") > 0 || vh_iget(item, "FINISH"))
@@ -691,8 +749,8 @@ setup_tasks(void)
     }
 
     /* Create user task steps */
-    vl_foreach(elt, tasks) {
-        task = vs_pget(elt);
+    v_iterate(tasks, i) {
+        task = vl_iter_pval(i);
         step = new_task(T_USER, task);
         vh_pstore(task, "STEP", step);
         add_task(step);
@@ -701,8 +759,8 @@ setup_tasks(void)
     /* Set up task 'follow' links */
     solver_msg(1, "Adding dependencies for task 'follow' entries");
 
-    vl_foreach(elt, tasks) {
-        task = vs_pget(elt);
+    v_iterate(tasks, i) {
+        task = vl_iter_pval(i);
         tstep = vh_pget(task, "STEP");
 
         if ((otask = vh_pget(task, "FOLLOW")) != NULL) {
@@ -710,6 +768,7 @@ setup_tasks(void)
             if (vh_exists(step, "NEXT") && vh_pget(step, "NEXT") != tstep)
                 err("more than one task needs to follow '%s' immediately",
                     vh_sgetref(otask, "DESC"));
+
             order_tasks(step, tstep);
             vh_pstore(step, "NEXT", tstep);
             vh_pstore(tstep, "PREV", step);
@@ -717,14 +776,14 @@ setup_tasks(void)
     }
 
     /* Process rooms and map connections */
-    vl_foreach(elt, rooms) {
-        room = vs_pget(elt);
+    v_iterate(rooms, i) {
+        room = vl_iter_pval(i);
         rlist = reachable_rooms(room);
 
         /* Flag 'before' tasks for this room as unsafe */
         if ((list = vh_pget(room, "BEFORE")) != NULL) {
-            vl_foreach(elt, list) {
-                task = vs_pget(elt);
+            v_iterate(list, j) {
+                task = vl_iter_pval(j);
                 step = vh_pget(task, "STEP");
                 vh_istore(step, "MODPATH", 1);
                 vh_sstore(step, "UNSAFE", "closes off room");
@@ -733,8 +792,8 @@ setup_tasks(void)
 
         /* Mention 'after' tasks for this room */
         if ((list = vh_pget(room, "AFTER")) != NULL) {
-            vl_foreach(elt, list) {
-                task = vs_pget(elt);
+            v_iterate(list, j) {
+                task = vl_iter_pval(j);
                 step = vh_pget(task, "STEP");
                 vh_istore(step, "MODPATH", 1);
                 add_task(step);
@@ -747,8 +806,8 @@ setup_tasks(void)
 
         /* Flag items which might have to be left before entering room */
         if ((list = vh_pget(room, "LEAVE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 vh_istore(item, "LEAVE", 1);
                 step = vh_pget(item, "STEP");
                 vh_istore(step, "MODPATH", 1);
@@ -757,8 +816,8 @@ setup_tasks(void)
 
         /* Must get required items before visiting this room */
         if ((list = vh_pget(room, "NEED")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 vh_istore(item, "USED", 1);
                 vh_istore(item, "NEEDED", 1);
 
@@ -772,8 +831,8 @@ setup_tasks(void)
             }
         }
 
-        vl_foreach(elt, rlist) {
-            reach = vs_pget(elt);
+        v_iterate(rlist, j) {
+            reach = vl_iter_pval(j);
 
             /* Invert reach 'leave' list if leaving all */
             if (vh_iget(reach, "LEAVEALL"))
@@ -781,8 +840,8 @@ setup_tasks(void)
 
             /* Flag items which might have to be left before using link */
             if ((list = vh_pget(reach, "LEAVE")) != NULL) {
-                vl_foreach(elt, list) {
-                    item = vs_pget(elt);
+                v_iterate(list, k) {
+                    item = vl_iter_pval(k);
                     vh_istore(item, "LEAVE", 1);
                     step = vh_pget(item, "STEP");
                     vh_istore(step, "MODPATH", 1);
@@ -791,8 +850,8 @@ setup_tasks(void)
 
             /* Flag 'before' tasks for this connection as unsafe */
             if ((list = vh_pget(reach, "BEFORE")) != NULL) {
-                vl_foreach(elt, list) {
-                    task = vs_pget(elt);
+                v_iterate(list, k) {
+                    task = vl_iter_pval(k);
                     step = vh_pget(task, "STEP");
                     vh_istore(step, "MODPATH", 1);
                     vh_sstore(step, "UNSAFE", "closes off link");
@@ -801,8 +860,8 @@ setup_tasks(void)
 
             /* Mention 'after' tasks for this connection */
             if ((list = vh_pget(reach, "AFTER")) != NULL) {
-                vl_foreach(elt, list) {
-                    task = vs_pget(elt);
+                v_iterate(list, k) {
+                    task = vl_iter_pval(k);
                     step = vh_pget(task, "STEP");
                     vh_istore(step, "MODPATH", 1);
                     add_task(step);
@@ -811,8 +870,8 @@ setup_tasks(void)
 
             /* Flag items which are needed for paths */
             if ((list = vh_pget(reach, "NEED")) != NULL) {
-                vl_foreach(elt, list) {
-                    item = vs_pget(elt);
+                v_iterate(list, k) {
+                    item = vl_iter_pval(k);
                     vh_istore(item, "USED", 1);
                     vh_istore(item, "NEEDED", 1);
                     step = vh_pget(item, "STEP");
@@ -827,8 +886,8 @@ setup_tasks(void)
     /* Process items */
     solver_msg(1, "Adding dependencies for item 'before/after' lists");
 
-    vl_foreach(elt, items) {
-        item = vs_pget(elt);
+    v_iterate(items, i) {
+        item = vl_iter_pval(i);
         istep = vh_pget(item, "STEP");
 
         /* Mark item as taken if it starts off that way */
@@ -837,8 +896,8 @@ setup_tasks(void)
 
         /* Must get required items before getting this item */
         if ((list = vh_pget(item, "NEED")) != NULL) {
-            vl_foreach(elt, list) {
-                oitem = vs_pget(elt);
+            v_iterate(list, j) {
+                oitem = vl_iter_pval(j);
                 vh_istore(oitem, "USED", 1);
 
                 room = vh_pget(oitem, "ROOM");
@@ -854,8 +913,8 @@ setup_tasks(void)
 
         /* Deal with 'after' tasks */
         if ((list = vh_pget(item, "AFTER")) != NULL) {
-            vl_foreach(elt, list) {
-                task = vs_pget(elt);
+            v_iterate(list, j) {
+                task = vl_iter_pval(j);
                 step = vh_pget(task, "STEP");
                 add_list(item, "RTASKS", step);
                 order_tasks(step, istep);
@@ -864,8 +923,8 @@ setup_tasks(void)
 
         /* Deal with 'before' tasks */
         if ((list = vh_pget(item, "BEFORE")) != NULL) {
-            vl_foreach(elt, list) {
-                task = vs_pget(elt);
+            v_iterate(list, j) {
+                task = vl_iter_pval(j);
                 step = vh_pget(task, "STEP");
                 add_list(item, "RTASKS", step);
                 order_tasks(istep, step);
@@ -876,36 +935,35 @@ setup_tasks(void)
     solver_msg(1, "Adding dependencies for task 'need/get/give/after' lists");
 
     /* Process task 'need/get/give/after/drop' stuff */
-    vl_foreach(elt, tasks) {
-        task = vs_pget(elt);
+    v_iterate(tasks, i) {
+        task = vl_iter_pval(i);
         tstep = vh_pget(task, "STEP");
+        first = first_task(tstep);
 
         /* Invert task 'drop' list if dropping all */
         if (vh_iget(task, "DROPALL"))
             invert_items(tstep, "DROP");
 
-        /* Must get required items before doing this task */
+        /* Must get required items before doing first task in follow-chain */
         if ((list = vh_pget(task, "NEED")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
                 vh_istore(item, "USED", 1);
 
-                room = vh_pget(item, "ROOM");
-                if (room != NULL) {
+                if ((room = vh_pget(item, "ROOM")) != NULL) {
                     get = vh_pget(item, "STEP");
                     order_tasks(get, tstep);
                 }
 
-                add_list(item, "TASKS", tstep);
+                add_list(item, "TASKS", first);
             }
         }
 
         /* Must do this task before giving or getting supplied items */
         if ((list = vh_pget(task, "GET")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
-                room = vh_pget(item, "ROOM");
-                if (room != NULL) {
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
+                if ((room = vh_pget(item, "ROOM")) != NULL) {
                     get = vh_pget(item, "STEP");
                     add_list(item, "RTASKS", tstep);
                     order_tasks(tstep, get);
@@ -914,10 +972,9 @@ setup_tasks(void)
         }
 
         if ((list = vh_pget(task, "GIVE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
-                room = vh_pget(item, "ROOM");
-                if (room != NULL) {
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
+                if ((room = vh_pget(item, "ROOM")) != NULL) {
                     get = vh_pget(item, "STEP");
                     add_list(item, "RTASKS", tstep);
                     order_tasks(tstep, get);
@@ -927,8 +984,8 @@ setup_tasks(void)
 
         /* Must do this task after all preceding tasks */
         if ((list = vh_pget(task, "AFTER")) != NULL) {
-            vl_foreach(elt, list) {
-                otask = vs_pget(elt);
+            v_iterate(list, j) {
+                otask = vl_iter_pval(j);
                 after = vh_pget(otask, "STEP");
                 order_tasks(after, tstep);
             }
@@ -938,17 +995,17 @@ setup_tasks(void)
     /* Process task 'lose' stuff */
     solver_msg(1, "Adding dependencies for task 'lose' lists");
 
-    vl_foreach(elt, tasks) {
-        task = vs_pget(elt);
+    v_iterate(tasks, i) {
+        task = vl_iter_pval(i);
         tstep = vh_pget(task, "STEP");
 
         if ((list = vh_pget(tstep, "LOSE")) != NULL) {
-            vl_foreach(elt, list) {
-                item = vs_pget(elt);
+            v_iterate(list, j) {
+                item = vl_iter_pval(j);
 
                 if ((itasks = vh_pget(item, "TASKS")) != NULL) {
-                    vl_foreach(elt, itasks) {
-                        step = vs_pget(elt);
+                    v_iterate(itasks, k) {
+                        step = vl_iter_pval(k);
                         order_tasks(step, tstep);
                     }
                 }
@@ -961,8 +1018,8 @@ setup_tasks(void)
     }
 
     /* Propagate 'unsafe' flags for 'follow' tasks */
-    vl_foreach(elt, tasks) {
-        task = vs_pget(elt);
+    v_iterate(tasks, i) {
+        task = vl_iter_pval(i);
         tstep = vh_pget(task, "STEP");
 
         if (vh_exists(tstep, "PREV") &&
@@ -981,8 +1038,8 @@ setup_tasks(void)
     while (flagged) {
         flagged = 0;
 
-        vl_foreach(elt, tasks) {
-            task = vs_pget(elt);
+        v_iterate(tasks, i) {
+            task = vl_iter_pval(i);
             tstep = vh_pget(task, "STEP");
             if (vh_defined(tstep, "UNSAFE"))
                 continue;
@@ -990,8 +1047,8 @@ setup_tasks(void)
             if ((list = vh_pget(tstep, "DO")) == NULL)
                 continue;
 
-            vl_foreach(elt, list) {
-                otask = vs_pget(elt);
+            v_iterate(list, j) {
+                otask = vl_iter_pval(j);
                 step = vh_pget(otask, "STEP");
                 if (vh_defined(step, "UNSAFE")) {
                     vh_sstore(tstep, "UNSAFE", "does unsafe task");
@@ -1008,7 +1065,7 @@ solve_game(void)
 {
     int count, tasksleft, status, ignore = 0;
     vhash *step, *trystep, *item, *next;
-    vscalar *elt;
+    viter iter;
 
     /* Don't bother if no tasks */
     if (tasklist == NULL || vl_length(tasklist) == 0)
@@ -1019,31 +1076,33 @@ solve_game(void)
     keep_unused_items = var_int("keep_unused_items");
 
     /* Build initial inventory */
-    vl_foreach(elt, items) {
-        item = vs_pget(elt);
+    v_iterate(items, iter) {
+        item = vl_iter_pval(iter);
         if (vh_pget(item, "ROOM") == NULL)
             vh_istore(item, "TAKEN", 1);
     }
 
     /* Process task list */
-    location = startroom;
+    MOVETO(startroom);
     next = NULL;
 
     solver_msg(0, "\nSolving game...");
 
     do {
-        solver_msg(1, "Location: %s", vh_sgetref(location, "DESC"));
+        solver_msg(1, "Location: %s", location_desc);
 
         /* Initialise path searches from this room */
-        init_path(location);
+        if (location != NULL)
+            init_path(location);
 
         /* Check for dropping unneeded items */
-        if (next == NULL && !vh_iget(location, "NODROP")) {
+        if (next == NULL &&
+            (location == NULL || !vh_iget(location, "NODROP"))) {
             while (1) {
                 count = 0;
 
-                vl_foreach(elt, items) {
-                    item = vs_pget(elt);
+                v_iterate(items, iter) {
+                    item = vl_iter_pval(iter);
 
                     /* Skip if not carried */
                     if (!vh_iget(item, "TAKEN"))
@@ -1072,8 +1131,8 @@ solve_game(void)
         step = NULL;
         tasksleft = 0;
 
-        vl_foreach(elt, tasklist) {
-            trystep = vs_pget(elt);
+        v_iterate(tasklist, iter) {
+            trystep = vl_iter_pval(iter);
 
             /* If task is done, skip it */
             if (vh_iget(trystep, "DONE"))
@@ -1100,7 +1159,6 @@ solve_game(void)
             if (status == TS_SAFE || all_tasks_safe) {
                 /* A safe task -- choose it */
                 step = trystep;
-                vl_break(tasklist);
                 break;
             } else if (step == NULL) {
                 /* The closest unsafe task */
@@ -1147,30 +1205,30 @@ task_graph(void)
 {
     char *before, *after;
     int count = 0;
-    vscalar *elt;
     vlist *list;
     vhash *step;
     V_BUF_DECL;
+    viter i, j;
     vgraph *g;
 
     g = vg_create();
 
     if (tasklist != NULL) {
-        vl_foreach(elt, tasklist) {
-            step = vs_pget(elt);
+        v_iterate(tasklist, i) {
+            step = vl_iter_pval(i);
             V_BUF_SET1("T%d", ++count);
             vh_sstore(step, "NODE", V_BUF_VAL);
             vg_node_pstore(g, V_BUF_VAL, step);
         }
 
-        vl_foreach(elt, tasklist) {
-            step = vs_pget(elt);
+        v_iterate(tasklist, i) {
+            step = vl_iter_pval(i);
             if ((list = vh_pget(step, "DEPEND")) == NULL)
                 continue;
 
             after = vh_sgetref(step, "NODE");
-            vl_foreach(elt, list) {
-                step = vs_pget(elt);
+            v_iterate(list, j) {
+                step = vl_iter_pval(j);
                 before = vh_sgetref(step, "NODE");
                 vg_link_oneway(g, before, after);
             }
@@ -1212,10 +1270,9 @@ task_status(vhash *room, vhash *step)
     taskroom = vh_pget(step, "ROOM");
     gotoroom = vh_pget(step, "GOTO");
 
-    if (taskroom != NULL && taskroom != room) {
+    if (taskroom != NULL && room != NULL && taskroom != room)
         if ((len = find_path(step, room, taskroom)) == NOPATH)
             return TS_INVALID;
-    }
 
     /* See whether task is safe */
     if (vh_exists(step, "SAFE")) {
@@ -1268,7 +1325,7 @@ want_item(vhash *item)
 {
     vhash *kitem, *task, *step;
     vlist *tasks, *kitems;
-    vscalar *elt;
+    viter iter;
 
     /* Yes if needed for movement */
     if (vh_iget(item, "NEEDED"))
@@ -1280,35 +1337,29 @@ want_item(vhash *item)
 
     /* Yes if at least one 'keep with' item is held */
     if ((kitems = vh_pget(item, "KEEP_WITH")) != NULL) {
-        vl_foreach(elt, kitems) {
-            kitem = vs_pget(elt);
-            if (!vh_exists(kitem, "TAKEN") || vh_iget(kitem, "TAKEN")) {
-                vl_break(kitems);
+        v_iterate(kitems, iter) {
+            kitem = vl_iter_pval(iter);
+            if (!vh_exists(kitem, "TAKEN") || vh_iget(kitem, "TAKEN"))
                 return 1;
-            }
         }
     }
 
     /* Yes if at least one 'keep until' task isn't done yet */
     if ((tasks = vh_pget(item, "KEEP_UNTIL")) != NULL) {
-        vl_foreach(elt, tasks) {
-            task = vs_pget(elt);
+        v_iterate(tasks, iter) {
+            task = vl_iter_pval(iter);
             step = vh_pget(task, "STEP");
-            if (!vh_iget(step, "DONE")) {
-                vl_break(tasks);
+            if (!vh_iget(step, "DONE"))
                 return 1;
-            }
         }
     }
 
     /* Yes if it's needed for at least one task */
     if ((tasks = vh_pget(item, "TASKS")) != NULL) {
-        vl_foreach(elt, tasks) {
-            task = vs_pget(elt);
-            if (!vh_iget(task, "DONE")) {
-                vl_break(tasks);
+        v_iterate(tasks, iter) {
+            task = vl_iter_pval(iter);
+            if (!vh_iget(task, "DONE"))
                 return 1;
-            }
         }
     }
 
@@ -1324,13 +1375,13 @@ warn_failure(void)
     vhash *step, *room, *reasons;
     vlist *list, *keys;
     int count = 0;
-    vscalar *elt;
     V_BUF_DECL;
+    viter iter;
 
     /* Record which tasks can't be done */
     reasons = vh_create();
-    vl_foreach(elt, tasklist) {
-        step = vs_pget(elt);
+    v_iterate(tasklist, iter) {
+        step = vl_iter_pval(iter);
         if (vh_iget(step, "DONE"))
             continue;
 
@@ -1351,10 +1402,7 @@ warn_failure(void)
         }
 
         /* Add it to list */
-        if ((list = vh_pget(reasons, V_BUF_VAL)) == NULL) {
-            list = vl_create();
-            vh_pstore(reasons, V_BUF_VAL, list);
-        }
+        list = vh_add_list(reasons, V_BUF_VAL);
 
         if (rdesc == NULL)
             V_BUF_SET(tdesc);
@@ -1368,17 +1416,23 @@ warn_failure(void)
     V_BUF_SET1("   final location:\n      %s\n",
                vh_sgetref(location, "DESC"));
 
-    keys = vh_sortkeys(reasons, NULL);
-    vl_foreach(elt, keys) {
-        reason = vs_sgetref(elt);
+    keys = vh_keys(reasons);
+
+    v_iterate(keys, iter) {
+        reason = vl_iter_svalref(iter);
+
         list = vh_pget(reasons, reason);
         entries = vl_join(list, ", ");
+
         list = vl_filltext(entries, 65);
         V_BUF_ADD1("   %s:\n", reason);
-        vl_foreach(elt, list)
-            V_BUF_ADD1("      %s\n", vs_sgetref(elt));
+
+        v_iterate(list, iter)
+            V_BUF_ADD1("      %s\n", vl_iter_svalref(iter));
     }
 
+    vl_destroy(keys);
+
     /* Print it */
     warn("can't solve game (%d task%s not done)\n%s",
          count, (count == 1 ? "" : "s"), V_BUF_VAL);
