@@ -1,10 +1,10 @@
 /*
- * This file is part of IFM, the Interactive Fiction Mapper,
- * copyright (C) G. Hutchings 1997-2004.
+ * This file is part of IFM (Interactive Fiction Mapper), copyright (C)
+ * Glenn Hutchings 1997-2008.
  *
- * IFM comes with ABSOLUTELY NO WARRANTY.  This is free
- * software, and you are welcome to redistribute it under
- * certain conditions; see the file COPYING for details.
+ * IFM comes with ABSOLUTELY NO WARRANTY.  This is free software, and you
+ * are welcome to redistribute it under certain conditions; see the file
+ * COPYING for details.
  */
 
 /* Variable function header */
@@ -32,6 +32,5 @@ extern void var_list(void);
 extern double var_real(char *id);
 extern void var_set(char *driver, char *id, vscalar *val);
 extern char *var_string(char *id);
-extern char *var_subst(char *string);
 
 #endif
